import re
from collections import OrderedDict

from rest_framework.renderers import UnicodeJSONRenderer

from util import PAGINATION_KEYS, get_instance_wrapper, get_list_wrapper


__authors__ = ["Vitaly Babiy", "Joakim Soderlund"]
__copyright__ = "Copyright (c) 2013, Vitaly Babiy"
__license__ = "BSD"


def underscore_to_camel(match):
	return match.group()[0] + match.group()[2].upper()


def camelize(data):
	if isinstance(data, dict):
		new_dict = OrderedDict()
		for key, value in data.items():
			new_key = re.sub(r"[a-z]_[a-z]", underscore_to_camel, key)
			new_dict[new_key] = camelize(value)
		return new_dict

	if isinstance(data, (list, tuple)):
		for i in range(len(data)):
			data[i] = camelize(data[i])
		return data

	return data


class EmberRenderer(UnicodeJSONRenderer):
	"""
	Renderer specifically designed for Ember.
	"""

	format = 'ember'
	media_type = 'application/ember'

	def render(self, data, accepted_media_type=None, renderer_context=None):
		if PAGINATION_KEYS == set(data.keys()):
			wrapper = get_list_wrapper(renderer_context)
			data = {
				'meta': {
					'count': data['count'],
					'next': data['next'],
					'previous': data['previous'],
				},
				wrapper: data['results'],
			}
		elif isinstance(data, (list, tuple)):
			wrapper = get_list_wrapper(renderer_context)
			data = {wrapper: data}
		else:
			wrapper = get_instance_wrapper(renderer_context)
			data = {wrapper: data}

		data = camelize(data)

		return super(EmberRenderer, self).render(data, accepted_media_type, renderer_context)
